# CMI Payment Gateway for Drupal Commerce

This module provides integration between Drupal Commerce 2.x and the Moroccan Centre Monétique Interbancaire (CMI) payment gateway, enabling secure online payments for Moroccan e-commerce websites.

## Features

- **3D Secure Authentication**: Full support for 3D Secure card authentication
- **Multiple Card Types**: Supports Visa, Mastercard, and Maestro cards
- **Secure Hash Validation**: SHA-512 hash validation for transaction security
- **Multi-language Support**: Supports Arabic, French, and English
- **Test Mode**: Built-in test mode for development and testing
- **Comprehensive Logging**: Detailed transaction logging for debugging
- **Server-to-Server Callbacks**: Secure callback handling for payment verification

## Requirements

- Drupal 9.5+ or Drupal 10.x
- PHP 8.0+
- Drupal Commerce 2.x
- Valid CMI merchant account and credentials

## Installation

1. Download and extract the module to `modules/custom/commerce_cmi` or install via Composer
2. Enable the module: `drush en commerce_cmi`
3. Clear caches: `drush cr`

## Configuration

### 1. CMI Merchant Setup

Contact Centre Monétique Interbancaire to obtain:
- Client ID (Merchant ID)
- Store Key (for hash generation)
- Gateway URL (test and production)

### 2. Drupal Configuration

1. Navigate to **Commerce → Configuration → Payment gateways**
2. Click **"Add payment gateway"**
3. Select **"CMI Payment Gateway"**
4. Configure the following settings:

#### Required Fields:
- **CMI Gateway URL**: The CMI payment endpoint
  - Test: `https://testpayment.cmi.co.ma/fim/est3Dgate`
  - Production: Provided by CMI
- **Client ID**: Your CMI merchant ID
- **Store Key**: Secret key for hash generation
- **Test Mode**: Enable for development/testing

#### Optional Configuration:
- **Display Name**: How the payment method appears to customers
- **Status**: Enable/disable the payment gateway

### 3. Currency Configuration

Ensure your store's default currency is supported by CMI. The module automatically handles currency code conversion for CMI (MAD = 504).

## Testing

### Test Credentials

CMI provides test credentials and test card numbers for development. Contact CMI support for:
- Test Client ID
- Test Store Key
- Test card numbers
- Test gateway URL

### Test Environment Setup

1. Enable **Test Mode** in the payment gateway configuration
2. Use the test gateway URL: `https://testpayment.cmi.co.ma/fim/est3Dgate`
3. Use CMI-provided test card numbers
4. Verify callback URL is accessible: `https://yoursite.com/commerce-cmi/callback`

## Payment Flow

1. **Customer Checkout**: Customer selects CMI payment method
2. **Redirect to CMI**: Customer is redirected to CMI's secure payment page
3. **Card Authentication**: Customer enters card details and completes 3D Secure
4. **Payment Processing**: CMI processes the payment
5. **Callback Notification**: CMI sends server-to-server callback to your site
6. **Return to Store**: Customer is redirected back to your store
7. **Order Completion**: Order status is updated based on payment result

## Security Features

### Hash Validation
- All requests use SHA-512 hash validation
- Hash includes all transaction parameters plus store key
- Prevents transaction tampering

### CSRF Protection
- Callback endpoint validates hash signatures
- Protects against unauthorized payment notifications

### Data Validation
- Order amount validation
- Currency validation
- Transaction ID validation

## Troubleshooting

### Common Issues

#### 1. Hash Validation Failures
- Verify Store Key is correct
- Check for extra spaces in configuration
- Ensure character encoding is UTF-8

#### 2. Callback Not Received
- Verify callback URL is publicly accessible
- Check server logs for callback requests
- Ensure CMI has correct callback URL

#### 3. Payment Redirects Fail
- Verify Gateway URL is correct
- Check for SSL certificate issues
- Validate all required parameters are sent

### Debugging

#### Enable Logging
The module logs all transactions to Drupal's logging system:
```bash
drush watchdog-show --type=commerce_payment
```

#### Check Recent Logs
```bash
drush watchdog-show --count=50 --filter=cmi
```

### Log Locations
- Payment attempts: `Reports → Recent log messages`
- Error details: Look for `commerce_payment` and `commerce_cmi` entries

## API Reference

### Payment Parameters

The module sends the following parameters to CMI:

| Parameter | Description | Required |
|-----------|-------------|----------|
| clientid | CMI Client ID | Yes |
| amount | Transaction amount | Yes |
| currency | Currency code (504 for MAD) | Yes |
| oid | Order ID | Yes |
| okUrl | Success return URL | Yes |
| failUrl | Failure return URL | Yes |
| callbackUrl | Server callback URL | Yes |
| TranType | Transaction type (PreAuth/PostAuth) | Yes |
| HASH | SHA-512 hash of parameters | Yes |

### Callback Parameters

CMI sends these parameters in callbacks:

| Parameter | Description |
|-----------|-------------|
| ProcReturnCode | Result code (00 = success) |
| AuthCode | Authorization code |
| oid | Order ID |
| amount | Transaction amount |
| HASH | Validation hash |

## Support

### CMI Support
- Email: integration.ecom@cmi.co.ma
- Documentation: Available in CMI integration package

### Module Support
- Issue tracking: [Module repository issues]
- Documentation: This README file

## Contributing

1. Fork the repository
2. Create a feature branch
3. Make your changes
4. Add tests if applicable
5. Submit a pull request

## License

This module is licensed under the GNU General Public License v2.0 or later.

## Changelog

### Version 2.0.0
- Drupal 10 compatibility
- Improved security and validation
- Enhanced error handling and logging
- Updated CMI API integration
- Better documentation

### Version 1.0.0
- Initial release
- Basic CMI integration
- Drupal 8 support
