<?php

namespace Drupal\commerce_cmi\Controller;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentGatewayInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Handles CMI payment gateway callbacks.
 */
class CmiCallbackController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a new CmiCallbackController object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Psr\Log\LoggerInterface $logger
   *   The logger.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, LoggerInterface $logger) {
    $this->entityTypeManager = $entity_type_manager;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('logger.channel.commerce_payment')
    );
  }

  /**
   * Handles CMI callback requests.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response.
   */
  public function cmiCallback(Request $request) {
    try {
      // Get POST data from CMI.
      $callback_data = $request->request->all();

      if (empty($callback_data)) {
        $this->logger->error('CMI callback received with no data');
        return new Response('APPROVED', 200);
      }

      $this->logger->info('CMI callback received with data: @data', [
        '@data' => json_encode($callback_data),
      ]);

      // Extract essential parameters.
      $oid = $callback_data['oid'] ?? '';
      $proc_return_code = $callback_data['ProcReturnCode'] ?? '';
      $amount = $callback_data['amount'] ?? '';

      if (empty($oid)) {
        $this->logger->error('CMI callback missing order ID');
        return new Response('APPROVED', 200);
      }

      // Load the order.
      $order_storage = $this->entityTypeManager->getStorage('commerce_order');
      $order = $order_storage->load($oid);

      if (!$order instanceof OrderInterface) {
        $this->logger->error('CMI callback: Order @oid not found', ['@oid' => $oid]);
        return new Response('APPROVED', 200);
      }

      // Find the payment gateway.
      $payment_gateway = $this->findPaymentGateway($order);
      if (!$payment_gateway) {
        $this->logger->error('CMI callback: No CMI payment gateway found for order @oid', ['@oid' => $oid]);
        return new Response('APPROVED', 200);
      }

      /** @var \Drupal\commerce_cmi\Plugin\Commerce\PaymentGateway\CmiRedirect $gateway_plugin */
      $gateway_plugin = $payment_gateway->getPlugin();

      // Validate the hash.
      if (!$this->validateHash($callback_data, $gateway_plugin)) {
        $this->logger->error('CMI callback hash validation failed for order @oid', ['@oid' => $oid]);
        return new Response('APPROVED', 200);
      }

      // Validate amount matches order total
      $order_total = $order->getTotalPrice();
      if (number_format((float) $amount, 2) !== number_format($order_total->getNumber(), 2)) {
        $this->logger->error('CMI callback amount mismatch for order @oid. Expected: @expected, Received: @received', [
          '@oid' => $oid,
          '@expected' => number_format($order_total->getNumber(), 2),
          '@received' => $amount,
        ]);
        return new Response('APPROVED', 200);
      }

      // Process the payment based on return code.
      if ($proc_return_code === '00') {
        // Successful transaction - update payment status and respond with POSTAUTH.
        $this->processSuccessfulPayment($order, $callback_data, $payment_gateway);
        $this->logger->info('CMI callback: Successful payment for order @oid', ['@oid' => $oid]);
        return new Response('ACTION=POSTAUTH', 200);
      } else {
        // Failed transaction - log the failure.
        $error_message = $callback_data['ErrMsg'] ?? 'Unknown error';
        $this->processFailedPayment($order, $callback_data, $payment_gateway);
        $this->logger->warning('CMI callback: Failed payment for order @oid. Error: @error (@code)', [
          '@oid' => $oid,
          '@error' => $error_message,
          '@code' => $proc_return_code,
        ]);
        return new Response('APPROVED', 200);
      }

    } catch (\Exception $e) {
      $this->logger->error('CMI callback error: @message', ['@message' => $e->getMessage()]);
      return new Response('APPROVED', 200);
    }
  }

  /**
   * Finds the CMI payment gateway for an order.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   *
   * @return \Drupal\commerce_payment\Entity\PaymentGatewayInterface|null
   *   The payment gateway or NULL.
   */
  protected function findPaymentGateway(OrderInterface $order) {
    $payment_gateway_storage = $this->entityTypeManager->getStorage('commerce_payment_gateway');
    $payment_gateways = $payment_gateway_storage->loadByProperties([
      'plugin' => 'cmi_redirect_checkout',
      'status' => TRUE,
    ]);

    return !empty($payment_gateways) ? reset($payment_gateways) : NULL;
  }

  /**
   * Validates the hash from CMI callback.
   *
   * @param array $data
   *   The callback data.
   * @param \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $gateway_plugin
   *   The payment gateway plugin.
   *
   * @return bool
   *   TRUE if hash is valid, FALSE otherwise.
   */
  protected function validateHash(array $data, OffsitePaymentGatewayInterface $gateway_plugin) {
    if (!isset($data['HASH'])) {
      return FALSE;
    }

    $received_hash = $data['HASH'];
    $configuration = $gateway_plugin->getConfiguration();
    $calculated_hash = $gateway_plugin->generateHash($data, $configuration['secret_key']);

    return hash_equals($received_hash, $calculated_hash);
  }

  /**
   * Processes a successful payment.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   * @param array $callback_data
   *   The callback data.
   * @param \Drupal\commerce_payment\Entity\PaymentGatewayInterface $payment_gateway
   *   The payment gateway.
   */
  protected function processSuccessfulPayment(OrderInterface $order, array $callback_data, PaymentGatewayInterface $payment_gateway) {
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');

    // Find existing payment or create new one.
    $payments = $payment_storage->loadByProperties([
      'order_id' => $order->id(),
      'payment_gateway' => $payment_gateway->id(),
    ]);

    if (!empty($payments)) {
      $payment = reset($payments);
    } else {
      $payment = $payment_storage->create([
        'state' => 'new',
        'amount' => $order->getTotalPrice(),
        'payment_gateway' => $payment_gateway->id(),
        'order_id' => $order->id(),
        'remote_id' => $callback_data['AuthCode'] ?? '',
      ]);
    }

    // Update payment status.
    $trantype = $callback_data['trantype'] ?? 'PreAuth';
    if ($trantype === 'PreAuth') {
      $payment->setState('authorization');
    } else {
      $payment->setState('completed');
    }

    $payment->setRemoteId($callback_data['AuthCode'] ?? '');
    $payment->setRemoteState($callback_data['ProcReturnCode']);
    $payment->save();
  }

  /**
   * Processes a failed payment.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The order.
   * @param array $callback_data
   *   The callback data.
   * @param \Drupal\commerce_payment\Entity\PaymentGatewayInterface $payment_gateway
   *   The payment gateway.
   */
  protected function processFailedPayment(OrderInterface $order, array $callback_data, PaymentGatewayInterface $payment_gateway) {
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');

    // Find existing payment.
    $payments = $payment_storage->loadByProperties([
      'order_id' => $order->id(),
      'payment_gateway' => $payment_gateway->id(),
    ]);

    if (!empty($payments)) {
      $payment = reset($payments);
      $payment->setState('authorization_failed');
      $payment->setRemoteState($callback_data['ProcReturnCode'] ?? '');
      $payment->save();
    }
  }

}
