<?php
namespace Drupal\commerce_cmi\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_price\Price;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Psr\Log\LoggerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the CMI payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "cmi_redirect_checkout",
 *   label = @Translation("CMI Payment Gateway"),
 *   display_label = @Translation("CMI - Centre Monétique Interbancaire"),
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_cmi\PluginForm\OffsiteRedirect\CmiForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "visa", "mastercard", "maestro"
 *   },
 * )
 */
class CmiRedirect extends OffsitePaymentGatewayBase {

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityTypeManagerInterface $entity_type_manager, PaymentTypeManager $payment_type_manager, PaymentMethodTypeManager $payment_method_type_manager, TimeInterface $time, LoggerInterface $logger) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_type_manager, $payment_type_manager, $payment_method_type_manager, $time);
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('plugin.manager.commerce_payment_type'),
      $container->get('plugin.manager.commerce_payment_method_type'),
      $container->get('datetime.time'),
      $container->get('logger.channel.commerce_payment')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    return [
      'api_url' => '',
      'api_key' => '',
      'secret_key' => '',
      'test_mode' => TRUE,
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['api_url'] = [
      '#type' => 'url',
      '#title' => $this->t('CMI Gateway URL'),
      '#description' => $this->t('The CMI payment gateway URL. Use https://testpayment.cmi.co.ma/fim/est3Dgate for testing.'),
      '#default_value' => $this->configuration['api_url'],
      '#required' => TRUE,
    ];

    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Client ID'),
      '#description' => $this->t('The Client ID provided by CMI.'),
      '#default_value' => $this->configuration['api_key'],
      '#required' => TRUE,
    ];

    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Store Key'),
      '#description' => $this->t('The Store Key provided by CMI for hash generation.'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];

    $form['test_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Test mode'),
      '#description' => $this->t('Enable test mode for development and testing.'),
      '#default_value' => $this->configuration['test_mode'],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::validateConfigurationForm($form, $form_state);

    if (!$form_state->getErrors() && $form_state->isSubmitted()) {
      $values = $form_state->getValue($form['#parents']);

      // Validate URL format.
      if (!filter_var($values['api_url'], FILTER_VALIDATE_URL)) {
        $form_state->setErrorByName('api_url', $this->t('Please enter a valid URL for the CMI Gateway.'));
      }

      // Validate required fields.
      if (empty($values['api_key'])) {
        $form_state->setErrorByName('api_key', $this->t('Client ID is required.'));
      }

      if (empty($values['secret_key'])) {
        $form_state->setErrorByName('secret_key', $this->t('Store Key is required.'));
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);
      $this->configuration['api_url'] = $values['api_url'];
      $this->configuration['api_key'] = $values['api_key'];
      $this->configuration['secret_key'] = $values['secret_key'];
      $this->configuration['test_mode'] = $values['test_mode'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onReturn(OrderInterface $order, Request $request) {
    $payment_storage = $this->entityTypeManager->getStorage('commerce_payment');

    try {
      // Get POST data from CMI.
      $response_data = $request->request->all();

      if (empty($response_data)) {
        throw new PaymentGatewayException('No response data received from CMI.');
      }

      // Validate the hash.
      if (!$this->validateHash($response_data)) {
        $this->logger->error('CMI payment hash validation failed for order @order_id. Response: @response', [
          '@order_id' => $order->id(),
          '@response' => json_encode($response_data),
        ]);
        throw new PaymentGatewayException('Invalid payment response from CMI.');
      }

      // Check if this is the correct order.
      $oid = $response_data['oid'] ?? '';
      if ($oid !== (string) $order->id()) {
        throw new PaymentGatewayException('Order ID mismatch in payment response.');
      }

      // Check transaction status.
      $proc_return_code = $response_data['ProcReturnCode'] ?? '';
      $trantype = $response_data['trantype'] ?? '';
      $amount = $response_data['amount'] ?? '';

      // Validate amount matches order total.
      $order_total = $order->getTotalPrice();
      if (number_format((float) $amount, 2) !== number_format($order_total->getNumber(), 2)) {
        throw new PaymentGatewayException('Amount mismatch between order and payment response.');
      }

      // Find existing payment or create new one.
      $payments = $payment_storage->loadByProperties([
        'order_id' => $order->id(),
        'payment_gateway' => $this->parentEntity->id(),
      ]);

      if (!empty($payments)) {
        $payment = reset($payments);
      } else {
        $payment = $payment_storage->create([
          'state' => 'new',
          'amount' => $order_total,
          'payment_gateway' => $this->parentEntity->id(),
          'order_id' => $order->id(),
          'remote_id' => $response_data['AuthCode'] ?? '',
        ]);
      }

      // Process payment based on result.
      if ($proc_return_code === '00') {
        // Successful payment
        if ($trantype === 'PreAuth') {
          $payment->setState('authorization');
        } else {
          $payment->setState('completed');
        }

        $payment->setRemoteId($response_data['AuthCode'] ?? '');
        $payment->setRemoteState($proc_return_code);
        $payment->save();

        $this->logger->info('CMI payment successful for order @order_id. Auth Code: @auth_code', [
          '@order_id' => $order->id(),
          '@auth_code' => $response_data['AuthCode'] ?? 'N/A',
        ]);

      } else {
        // Failed payment.
        $error_message = $response_data['ErrMsg'] ?? 'Payment failed';
        $payment->setState('authorization_failed');
        $payment->setRemoteState($proc_return_code);
        $payment->save();

        $this->logger->warning('CMI payment failed for order @order_id. Error: @error (@code)', [
          '@order_id' => $order->id(),
          '@error' => $error_message,
          '@code' => $proc_return_code,
        ]);

        throw new PaymentGatewayException('Payment failed: ' . $error_message);
      }

    } catch (\Exception $e) {
      $this->logger->error('CMI payment processing error for order @order_id: @error', [
        '@order_id' => $order->id(),
        '@error' => $e->getMessage(),
      ]);
      throw new PaymentGatewayException($e->getMessage());
    }
  }

  /**
   * {@inheritdoc}
   */
  public function onCancel(OrderInterface $order, Request $request) {
    $this->logger->info('CMI payment cancelled for order @order_id', [
      '@order_id' => $order->id(),
    ]);

    $this->messenger()->addWarning($this->t('Payment was cancelled. You may try again or choose a different payment method.'));
  }

  /**
   * Validates the hash from CMI response.
   *
   * @param array $data
   *   The response data from CMI.
   *
   * @return bool
   *   TRUE if hash is valid, FALSE otherwise.
   */
  private function validateHash(array $data) {
    if (!isset($data['HASH'])) {
      return FALSE;
    }

    $received_hash = $data['HASH'];
    $calculated_hash = $this->generateHash($data, $this->configuration['secret_key']);

    return hash_equals($received_hash, $calculated_hash);
  }

  /**
   * Generates hash for CMI transaction.
   *
   * @param array $data
   *   The transaction data.
   * @param string $store_key
   *   The store key.
   *
   * @return string
   *   The generated hash.
   */
  public function generateHash(array $data, $store_key) {
    $post_params = array_keys($data);
    natcasesort($post_params);

    $hash_val = '';
    foreach ($post_params as $param) {
      $param_value = html_entity_decode(preg_replace("/\n$/", "", $data[$param]), ENT_QUOTES, 'UTF-8');
      $escaped_param_value = str_replace("|", "\\|", str_replace("\\", "\\\\", $param_value));

      $lower_param = strtolower($param);
      if ($lower_param !== 'hash' && $lower_param !== 'encoding') {
        $hash_val .= $escaped_param_value . '|';
      }
    }

    $escaped_store_key = str_replace("|", "\\|", str_replace("\\", "\\\\", $store_key));
    $hash_val .= $escaped_store_key;

    $calculated_hash_value = hash('sha512', $hash_val);
    return base64_encode(pack('H*', $calculated_hash_value));
  }

}


