<?php

namespace Drupal\commerce_cmi\PluginForm\OffsiteRedirect;

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm;
use Drupal\commerce_price\Entity\Currency;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;

/**
 * Provides the CMI payment offsite form.
 */
class CmiForm extends PaymentOffsiteForm {

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildConfigurationForm($form, $form_state);

    /** @var \Drupal\commerce_payment\Entity\PaymentInterface $payment */
    $payment = $this->entity;
    /** @var \Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayInterface $payment_gateway_plugin */
    $payment_gateway_plugin = $payment->getPaymentGateway()->getPlugin();
    $configuration = $payment_gateway_plugin->getConfiguration();
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
    $order = $payment->getOrder();

    // Validate configuration.
    if (empty($configuration['api_url']) || empty($configuration['api_key']) || empty($configuration['secret_key'])) {
      $this->messenger()->addError($this->t('CMI payment gateway is not properly configured. Please contact the site administrator.'));
      return $form;
    }

    $total_price = $order->getTotalPrice();
    $currency = Currency::load($total_price->getCurrencyCode());

    // Generate unique random number for this transaction.
    $rnd = microtime();

    // Get current language.
    $current_lang = \Drupal::languageManager()->getCurrentLanguage()->getId();
    $lang_map = [
      'en' => 'en',
      'fr' => 'fr',
      'ar' => 'ar',
    ];
    $cmi_lang = $lang_map[$current_lang] ?? 'fr';

    // Prepare billing address
    $billing_profile = $order->getBillingProfile();
    $billing_address = null;
    if ($billing_profile && $billing_profile->hasField('address') && !$billing_profile->get('address')->isEmpty()) {
      $billing_address = $billing_profile->get('address')->first();
    }

    // Prepare payment data according to CMI API.
    $data = [
      'clientid' => $configuration['api_key'],
      'amount' => number_format($total_price->getNumber(), 2, '.', ''),
      'okUrl' => $form['#return_url'],
      'failUrl' => $form['#cancel_url'],
      'TranType' => 'PreAuth', // Use PreAuth as default, can be configured later.
      'callbackUrl' => Url::fromRoute('commerce_cmi.callback', [], ['absolute' => TRUE])->toString(),
      'shopurl' => Url::fromRoute('<front>', [], ['absolute' => TRUE])->toString(),
      'currency' => $currency ? $currency->getNumericCode() : '504', // Default to MAD (504).
      'rnd' => $rnd,
      'storetype' => '3D_PAY_HOSTING',
      'hashAlgorithm' => 'ver3',
      'lang' => $cmi_lang,
      'refreshtime' => '5',
      'encoding' => 'UTF-8',
      'oid' => (string) $order->id(),
    ];

    // Add billing information if available.
    if ($billing_address) {
      $data['BillToName'] = trim(($billing_address->getGivenName() . ' ' . $billing_address->getFamilyName()));
      $data['BillToCompany'] = $billing_address->getOrganization() ?: '';
      $data['BillToStreet1'] = trim(($billing_address->getAddressLine1() . ' ' . $billing_address->getAddressLine2()));
      $data['BillToCity'] = $billing_address->getLocality() ?: '';
      $data['BillToStateProv'] = $billing_address->getAdministrativeArea() ?: '';
      $data['BillToPostalCode'] = $billing_address->getPostalCode() ?: '';
      $data['BillToCountry'] = $billing_address->getCountryCode() ?: 'MA';
    } else {
      // Default values if no billing address.
      $data['BillToName'] = 'Customer';
      $data['BillToCompany'] = '';
      $data['BillToStreet1'] = '';
      $data['BillToCity'] = '';
      $data['BillToStateProv'] = '';
      $data['BillToPostalCode'] = '';
      $data['BillToCountry'] = 'MA';
    }

    // Add customer email.
    $data['email'] = $order->getEmail() ?: '';

    // Add optional fields.
    $data['tel'] = '';
    $data['symbolCur'] = $total_price->getCurrencyCode();

    // Generate hash.
    $data['HASH'] = $payment_gateway_plugin->generateHash($data, $configuration['secret_key']);

    // Log transaction attempt for debugging.
    \Drupal::logger('commerce_cmi')->info('CMI payment initiated for order @order_id with amount @amount @currency', [
      '@order_id' => $order->id(),
      '@amount' => $data['amount'],
      '@currency' => $data['currency'],
    ]);

    return $this->buildRedirectForm($form, $form_state, $configuration['api_url'], $data, 'post');
  }

}
